(***********************************************************************)
(*                                                                     *)
(*                             Pio (OCaml)                             *)
(*                                                                     *)
(*                    Polyglot Input/Output Library                    *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright 2005-2014 INRIA.                                         *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: read_communication.ml,v 1.14 2014/02/09 20:13:38 fclement Exp $ *)

open Typed_value;;
open Communication;;

let debug fun_name =
  Options_pio.pf.Udebug.debug
    (Printf.sprintf "Read_communication.%s" fun_name)
;;

let error fun_name msg =
  Options_pio.pf.Udebug.error
    (Printf.sprintf "Read_communication.%s" fun_name) msg;
  exit 2
;;

let r_values ib n =
  debug "r_values" "start reading";
  if n = 0 then
    begin
      debug "r_values" "0 value(s) read";
      [| |]
    end
  else
    begin
      let vals = Array.make n (Scalar (Sint 0)) in
      for i = 0 to n - 1 do
        vals.(i) <- Read_typed_value.read_typed_value ib
      done;
      debug "r_values" (Printf.sprintf "%i value(s) read" n);
      vals
    end
;;

let unsafe_read_communication ib =
  debug "read_communication" "start reading";
  Scanf.bscanf ib "(\n" ();
  debug "read_communication" "\"(\" marker read";
  Scanf.bscanf ib "%%%c <%d> " (fun t n ->
    let com =
      match t with
      | 'p' ->
          debug "read_communication" "phrase tag read";
          Phrase (r_values ib n)
      | 't' ->
          debug "read_communication" "task tag read";
          Scanf.bscanf ib "%S\n" (fun task_name ->
            debug "read_communication" "task name read";
            let vals = r_values ib n in
            Task (task_name, vals))
      | 'r' ->
          debug "read_communication" "result tag read";
          Result (r_values ib n)
      | 'e' ->
          debug "read_communication" "error tag read";
          Error (r_values ib n)
      | 's' ->
          debug "read_communication" "service tag read";
          Scanf.bscanf ib "%S\n" (fun sname ->
            debug "read_communication" "service name read";
            let service_name = service_name_of_string sname in
            let vals = r_values ib n in
            Service (service_name, vals))
      | c ->
        raise
          (Wrong_communication
             (Printf.sprintf "Unknown communication kind ``%%%c''" c)) in
    Scanf.bscanf ib "%0c" (function
      | (* ( *) ')' ->
        begin
          Scanf.bscanf ib ")\n\n" ();
          debug "read_communication" "\")\" marker read";
          debug "read_communication" "finish reading";
          com
        end
      | c ->
        raise
          (Wrong_communication
            (Printf.sprintf
               "Unknown end of communication character ``%c''" c))))
;;

let where_are_we ib = Scanf.bscanf ib "%l%n" (fun l n -> (l, n));;

let wrong_communication_msg ib =
  let fname = Scanf.Scanning.name_of_input ib in
  let (line, char) = where_are_we ib in
  Printf.sprintf
    "Wrong communication detected at line %d character %d in source %S"
    line char fname
;;

let read_communication ib =
  try unsafe_read_communication ib with
  (* Exceptions from module [Scanf] are caught here to return a service
     communication reporting the error. *)
  | Wrong_typed_value _
  | Scanf.Scan_failure _
  | Failure _
  | End_of_file as x ->
      let msg = wrong_communication_msg ib in
      raise
        (Wrong_communication
          (Printf.sprintf "%s:\n%s" msg (Printexc.to_string x)))
  (* Invalid_argument s reveals an error in the implementation of the library
     and should not be caught. *)
  | Wrong_communication _ as exn -> raise exn
  | x -> error "read_communication" (Printexc.to_string x)
;;
