(***********************************************************************)
(*                                                                     *)
(*                              Ref-indic                              *)
(*                                                                     *)
(*                        Refinement indicators                        *)
(*                                                                     *)
(*               EPI Pomdapi, INRIA Paris - Rocquencourt               *)
(*                                                                     *)
(*  Copyright (c) 2013-2014 INRIA.                                     *)
(*  Institut National de Recherche en Informatique et en Automatique.  *)
(*  All rights reserved. This file is distributed only by permission.  *)
(*                                                                     *)
(*  Hend Ben Ameur <hbenameur@yahoo.ca>                                *)
(*  Francois Clement <Francois.Clement@inria.fr>                       *)
(*  Pierre Weis <Pierre.Weis@inria.fr>                                 *)
(*                                                                     *)
(***********************************************************************)

(* $Id: test_model.ml,v 1.1 2014/06/17 14:58:34 fclement Exp $ *)

let matrix_get_line m = Array.get (Umatrix.to_array_array m);;

let matrix_map_line2 f m1 m2 =
  let aa1 = Umatrix.to_array_array m1
  and aa2 = Umatrix.to_array_array m2 in
  Umatrix.of_array_array (Uarray.map2 f aa1 aa2)
;;

module Nat = Unatural;;
module Cmd_opt = Ucommand_option;;

open Wire_types_print;;

let ppf = Format.err_formatter;;

let print_nat = Unatural_print.print_t;;
let print_data = Umatrix_print.print_t Lib_print.print_quoted_float;;

let print fun_name ident p_value value =
  Format.fprintf ppf "@[<hv 2>%s: %s =@ %a@]@." fun_name ident p_value value
;;

let print_message = Format.fprintf ppf "@[%s@]@.";;

let print_exc f x =
  try f x with exc -> print_message (Printexc.to_string exc)
;;

let print_newline () = Format.fprintf ppf "@.";;

let print_init dim get_data sz ppm_1 maa_1 j_1 =
  print "init" "dim" print_nat dim;
  print "init" "dm" print_data (get_data ());
  print "init" "sz" print_size sz;
  print "init" "ppm_1" print_partition ppm_1;
  print "init" "maa_1" print_coarse_parameter maa_1;
  print "init" "j_1" print_cost j_1;
;;

let print_grad spm pm sgm gm =
  print "grad" spm print_fine_parameter pm;
  print "grad" sgm print_fine_gradient gm;
;;

let print_optim sppm ppm smaa_init maa_init smaa_opt maa_opt sj_opt j_opt =
  print "optim" sppm print_partition ppm;
  print "optim" smaa_init print_coarse_parameter maa_init;
  print "optim" smaa_opt print_coarse_parameter maa_opt;
  print "optim" sj_opt print_cost j_opt;
;;

let scalar_split_first_part ga ppa =
  let nbz = Nat.succ (Array.fold_left max Nat._0_ ppa) in
  let add_zone i pp = if pp = Nat._0_ && ga.(i) < 0. then nbz else pp in
  Array.mapi add_zone ppa
;;

let split_first_component maa =
  let dim = Uarray.length maa in
  let line_dupl_first i =
    let maai = maa.(i) in
    let nbzi = Uarray.length maai in
    let dupl_first j = if j < Nat.to_int nbzi then maai.(j) else maai.(0) in
    Uarray.init (Nat.succ nbzi) dupl_first in
  Uarray.init dim line_dupl_first
;;

(* Pseudo-vector dimension management:
   first zone is split according to first component. *)

let psv_split_first_part gm ppm =
  let gm0 = matrix_get_line gm 0 in
  let ppm0 = matrix_get_line ppm 0 in
  let ppm0' = scalar_split_first_part gm0 ppm0 in
  Umatrix.map_ij (fun _i j _ -> ppm0'.(j)) ppm
;;

let psv_split_first_zone gm_i ppm_i maa_i =
  let ppm_ip1 = psv_split_first_part gm_i ppm_i in
  let maa_ip1_init = split_first_component maa_i in
  (ppm_ip1, maa_ip1_init)
;;

(* Pseudo-best-component-for-each dimension management:
   first zone is split in each component. *)

let psbcfe_split_first_part gm ppm =
  matrix_map_line2 scalar_split_first_part gm ppm
;;

let psbcfe_split_first_zone gm_i ppm_i maa_i =
  let ppm_ip1 = psbcfe_split_first_part gm_i ppm_i in
  let maa_ip1_init = split_first_component maa_i in
  (ppm_ip1, maa_ip1_init)
;;

(* Test. *)

let test_init get_data get_dim init =
  let dim = get_dim () in
  let (sz, ppm_1, maa_1, j_1) = init dim in
  print_init dim get_data sz ppm_1 maa_1 j_1;
  print_newline ();
  (sz, ppm_1, maa_1, j_1)
;;

let test_grad coarse_to_fine grad ppm_i maa_i spm_i sgm_i =
  let pm_i = coarse_to_fine ppm_i maa_i in
  let gm_i = grad pm_i in
  print_grad spm_i pm_i sgm_i gm_i;
  print_newline ();
  gm_i
;;

let test_optim optim ppm_ip1 maa_ip1_init
    sppm_ip1 smaa_ip1_init smaa_ip1 sj_ip1 =
  let (maa_ip1, j_ip1) = optim (ppm_ip1, maa_ip1_init) in
  print_optim
    sppm_ip1 ppm_ip1 smaa_ip1_init maa_ip1_init
    smaa_ip1 maa_ip1 sj_ip1 j_ip1;
  print_newline ();
  (maa_ip1, j_ip1)
;;

let test_iter split_first_zone coarse_to_fine grad optim ppm_i maa_i
    spm_i sgm_i sppm_ip1 smaa_ip1_init smaa_ip1 sj_ip1 =
  let gm_i = test_grad coarse_to_fine grad ppm_i maa_i spm_i sgm_i in
  let (ppm_ip1, maa_ip1_init) = split_first_zone gm_i ppm_i maa_i in
  let (maa_ip1, j_ip1) =
    test_optim optim ppm_ip1 maa_ip1_init
      sppm_ip1 smaa_ip1_init smaa_ip1 sj_ip1 in
  (ppm_ip1, maa_ip1, j_ip1)
;;

let test_dm print_header_dm split_dm coarse_to_fine grad optim ppm_1 maa_1 =
  print_header_dm ();
  let test_iter = test_iter split_dm coarse_to_fine grad optim in
  let (ppm_2, maa_2, _j_2) =
    test_iter ppm_1 maa_1 "pm_1" "gm_1" "ppm_2" "maa_2_init" "maa_2" "j_2" in
  let (ppm_3, maa_3, j_3) =
    test_iter ppm_2 maa_2 "pm_2" "gm_2" "ppm_3" "maa_3_init" "maa_3" "j_3" in
  (ppm_3, maa_3, j_3)
;;

let print_header_vector () =
  print_message "Pseudo-vector dimension management:";
  print_message "  first zone is split according to first component.";
  print_newline ();
;;

let test_vector = test_dm print_header_vector psv_split_first_zone;;

let print_header_best_component_for_each () =
  print_message "Pseudo-best-component-for-each dimension management:";
  print_message "  first zone is split in each component.";
  print_newline ();
;;

let test_best_component_for_each =
  test_dm print_header_best_component_for_each psbcfe_split_first_zone
;;

let ctof = Worker.coarse_parameter_to_fine_parameter;;
let task_engines =
  {
   Worker.init_engine = Model.init_engine Worker.check_valid_dimension;
   grad_engine = Model.grad_engine;
   optim_engine = Model.optim_engine ctof;
   final_engine = Model.final_engine;
  }
;;

let { Worker.init = init;
      grad = grad;
      optim = optim;
      final = final; } = Worker.make_tasks task_engines
;;

let main () =
  Cmd_opt.parse ();
  let pf = Pio.worker_make_printing_facilities () in
  let spf = Udebug.set_function_name "" pf in
  let init = init spf in
  let grad = grad spf in
  let optim = optim spf in
  let _final = final spf in
  let (_sz, ppm_1, maa_1, _j_1) =
    test_init Model.get_data Model.get_dim init in
  ignore (test_vector ctof grad optim ppm_1 maa_1);
  ignore (test_best_component_for_each ctof grad optim ppm_1 maa_1);
;;

main ();;
