function display_transmissivity(FNAME, NX, NY, MAX_VALUE)
%DISPLAY_TRANSMISSIVITY   Display transmissivity distributions.
%   DISPLAY_TRANSMISSIVITY(FNAME, NX, NY, MAX_VALUE) displays the
%   two-dimensional transmissivity distribution stored in data file FNAME
%   with profile NX x NY. Values are clipped below 0 and above MAX_VALUE.
%
%   Format of data files is:
%    * one number per line,
%    * first the total number of values,
%    * and then the values line-wise.

% Check argument numbers.
if ~(nargin == 4)
  error('displayTransmissivity:wrongArgumentNumber', ...
        'Exactly three input argument');
end
if ~(nargout == 0)
  error('displayTransmissivity:wrongArgumentNumber', ...
        'No output argument');
end

% Check argument types.
if ~ischar(FNAME)
  error('displayTransmissivity:wrongType', ...
        'First argument must be a character string');
end
if ~(isnumeric(NX) & isnumeric(NY) & isnumeric(MAX_VALUE))
  error('displayTransmissivity:wrongType', ...
        'Second, third, and fourth arguments must be numbers');
end

% Check argument dimensions.
if ~(length(FNAME) == prod(size(FNAME)))
  error('displayTransmissivity:wrongDimension', ...
        'First argument must be a vector');
end
if ~((length(NX) == 1) & (length(NY) == 1) & (length(MAX_VALUE) == 1))
  error('displayTransmissivity:wrongDimension', ...
        'Second, third, and fourth arguments must be scalars');
end

% Check data file existence.
if ~exist(FNAME, 'file')
  error('displayTransmissivity:fileNotFound', ...
        'File %s is not found', FNAME);
end

% Load data file contents.
tab = load(FNAME);

% Check file format.
len = tab(1);
if ~(length(tab) == len + 1)
  error('displayTransmissivity:wrongFormat', ...
        'First value is not the length of the rest of the data file');
end

% Check arguments compatibility.
if ~(len == NX * NY)
  error('displayTransmissivity:wrongProfileSpecification', ...
        [ 'Profile specification %d x %d' ...
          ' is not compatible with the length of the data read %d' ], ...
        NX, NY, len);
end

% Matrix form (line-wise).
t = reshape(tab(2:end), NX, NY)'

% Display (matrix is flipped up-down when displayed and cells are given the
% value of their lower-left corner).
tt = zeros(NY+1, NX+1);
tt(1:NY, 1:NX) = flipud(t);
figure(1);
pcolor(tt);
axis square;
view(2);
%shading flat;
colormap('cool');
caxis([ 0 MAX_VALUE ]);
colorbar;

print('-dpng', '-r72', [ FNAME '.png' ]);

%display_transmissivity('small_transmissivity_data', 2, 2, 20)
%display_transmissivity('medium_transmissivity_data', 10, 10, 20)
%display_transmissivity('fracture_transmissivity_data', 10, 10, 200)
