CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC
C                                                                     C
C                            Pio (Fortran)                            C
C                                                                     C
C                    Polyglot Input/Output Library                    C
C                                                                     C
C               EPI Pomdapi, INRIA Paris - Rocquencourt               C
C                                                                     C
C  Copyright 2005-2014 INRIA.                                         C
C  Institut National de Recherche en Informatique et en Automatique.  C
C  All rights reserved. This file is distributed only by permission.  C
C                                                                     C
C  Francois Clement <Francois.Clement@inria.fr>                       C
C  Pierre Weis <Pierre.Weis@inria.fr>                                 C
C  Hend Ben Ameur <hbenameur@yahoo.ca>                                C
C                                                                     C
CCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC

      
C $Id: scan_typed_value.fpp,v 1.5 2014/03/13 20:08:47 fclement Exp $
      
C     
C     Routines to scan a typed value.
C     
C     Value ::= Scalar | Array
C     Array ::= Vector | Multi_D
C     Vector ::= "[1" '\n' Size '\n' Item* "1]" '\n'
C     Multi_D ::=
C        '[' Natural p '\n'
C        '<' Natural_list '>' '\n'
C        Array_layout '\n'
C        Row*
C        Natural p ']' '\n'
C     Natural_list ::= Natural | Natural ", " Natural_list
C     Array_layout ::= Natural_list | 'C' | 'F'
C     Row ::= "[|" '\n' Item* "|];" '\n'
C     Item ::= Scalar ';' '\n'
C     Scalar ::= Simple_lexem | Tuple_lexem
C     Tuple_lexem ::=
C        | '(' Simple_lexem ", " Simple_lexem ')'
C        | '(' Simple_lexem ", " Simple_lexem ", " Simple_lexem ')'
C     Simple_lexem ::= String_lexem | Number_lexem
C     String_lexem ::= Size '\n' String
C     Size ::= '<' Natural '>'
C     Natural ::= Decimal_digit+
C     String ::= '"' Char* '"'
C     Char ::= Escaped_char | Printable_char
C     Escaped_char ::= '\\' Decimal_digit Decimal_digit Decimal_digit
C     Decimal_digit ::=
C        '0' | '1' | '2' | '3' | '4' | '5' | '6' | '7' | '8' | '9'
C     Printable_char ::= '[32-33]' | '[35-91]' | '[93-126]'
C     Number_lexem ::= Integer | Float | Binary_float
C     Integer ::= ``a valid OCaml lexem: an integer''
C     Float ::= ``a valid OCaml lexem: a floating point number''
C     Binary_float ::= 8 bytes
      
#include "pio_constants.h"
      
      integer function scan_size()
C     Scan a Size in the sense of the PIO library.
      
      implicit none
      
C     Local variable.
      character sbr*9
      
C     External functions.
      logical pio_get_debug_mode
      integer scan_natural
      
      sbr = 'scan_size'
      
      call verify_scan_current_char_and_advance(sbr, '<')
      scan_size = scan_natural(sbr)
      call verify_scan_current_char_and_advance(sbr, '>')
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'size ', scan_size, ' scanned.'
      endif
      
      return
      end
      
      
      subroutine scan_string_value(ic)
C     Scan a string value.
      
      implicit none
      
C     Input argument.
      integer ic
      
C     Local variables.
      character sbr*17, s*MAX_STRING_LENGTH
      integer lenv
      
C     External functions.
      character get_stringbuf*MAX_STRING_LENGTH
      integer scan_size
      
      sbr = 'scan_string_value'
      
      lenv = scan_size()
      
      call fill_buffer(sbr, ic)
      call scan_string(sbr, lenv)
      
      s = get_stringbuf(sbr, lenv)
      
      call seq_store_string_value(sbr, PIO_IN, s, lenv)
      
      return
      end
      
      
      subroutine scan_int_value()
C     Scan an integer value.
      
      implicit none
      
C     Local variables.
      character sbr*14
      integer i
      
C     External function.
      integer scan_int
      
      sbr = 'scan_int_value'
      
      i = scan_int(sbr)
      call seq_store_int_value(sbr, PIO_IN, i)
      
      return
      end
      
      
      subroutine scan_float_value()
C     Scan a float value.
      
      implicit none
      
C     Local variables.
      character sbr*16
      FLOAT f
      
C     External functions.
      logical encoding_mode_is_binary
      FLOAT scan_binary_float, scan_ascii_float
      
      sbr = 'scan_float_value'
      
      if (encoding_mode_is_binary()) then
         f = scan_binary_float(sbr)
      else
         f = scan_ascii_float()
      endif
      
      call seq_store_float_value(sbr, PIO_IN, f)
      
      return
      end
      
      
      subroutine scan_simple(ic, typ)
C     Scan a simple value.
      
      implicit none
      
C     Input arguments.
      integer ic, typ
      
C     Local variables.
      character sbr*11
      
      sbr = 'scan_simple'
      
      if (typ .eq. SSTRING) then
         
         call scan_string_value(ic)
         
      else if (typ .eq. SINT) then
         
         call scan_int_value()
         
      else if (typ .eq. SFLOAT) then
         
         call scan_float_value()
         
      else
         
C     impossible, already checked in scan_scalar
         write(PIO_STDERR, *) '[Pio/Fortran] typ = ', typ
         call error_internal_error(sbr)
         
      endif
      
      return
      end
      
      
      subroutine scan_couple(ic, typ)
C     Scan a couple value of type typ.
      
      implicit none
      
C     Input arguments.
      integer ic, typ
      
C     Local variables.
      character sbr*11
      integer item_typ
      
C     External function.
      logical is_ascii_float
      
      sbr = 'scan_couple'
      
      item_typ = MAKE_COUPLE_ITEM_TYPE(typ)
      
      call verify_scan_current_char_and_advance(sbr, '(')
      
      if (is_ascii_float(typ)) then
         call fill_floatbuf(sbr, ',')
      endif
      call scan_simple(ic, item_typ)
      
      call verify_scan_current_string_and_advance(sbr, ', ', 2)
      
      if (is_ascii_float(typ)) then
         call fill_floatbuf(sbr, ')')
      endif
      call scan_simple(ic, item_typ)
      
      call verify_scan_current_char_and_advance(sbr, ')')
      
      return
      end
      
      
      subroutine scan_triple(ic, typ)
C     Scan a triple value of type typ.
      
      implicit none
      
C     Input arguments.
      integer ic, typ
      
C     Local variables.
      character sbr*11
      integer item_typ
      
C     External function.
      logical is_ascii_float
      
      sbr = 'scan_triple'
      
      item_typ = MAKE_TRIPLE_ITEM_TYPE(typ)
      
      call verify_scan_current_char_and_advance(sbr, '(')
      
      if (is_ascii_float(typ)) then
         call fill_floatbuf(sbr, ',')
      endif
      call scan_simple(ic, item_typ)
      
      call verify_scan_current_string_and_advance(sbr, ', ', 2)
      
      if (is_ascii_float(typ)) then
         call fill_floatbuf(sbr, ',')
      endif
      call scan_simple(ic, item_typ)
      
      call verify_scan_current_string_and_advance(sbr, ', ', 2)
      
      if (is_ascii_float(typ)) then
         call fill_floatbuf(sbr, ')')
      endif
      call scan_simple(ic, item_typ)
      
      call verify_scan_current_char_and_advance(sbr, ')')
      
      return
      end
      
      
      subroutine scan_scalar(ic, typ)
C     Scan a scalar value.
      
      implicit none
      
C     Input arguments.
      integer ic, typ
      
C     Local variable.
      character sbr*11
      
C     External functions.
      logical is_scalar_simple_type, is_ascii_float
      logical is_scalar_couple_type
      logical is_scalar_triple_type
      
      sbr = 'scan_scalar'
      
      call fill_buffer(sbr, ic)
      
      if (is_scalar_simple_type(typ)) then
         if (is_ascii_float(typ)) then
            call fill_floatbuf(sbr, ';')
         endif
         call scan_simple(ic, typ)
      else if (is_scalar_couple_type(typ)) then
         call scan_couple(ic, typ)
      else if (is_scalar_triple_type(typ)) then
         call scan_triple(ic, typ)
      else
         write(PIO_STDERR, *) '[Pio/Fortran] typ = ', typ
         call error_assertion_failure(sbr,'wrong type checking.')
      endif
      
      return
      end
      
      
      subroutine scan_scalar_item(ic, typ)
C     Scan an item of an array.
      
      implicit none
      
C     Input arguments.
      integer ic, typ
      
C     Local variable.
      character sbr*16, c
      
C     External function.
      character int_to_char
      
      sbr = 'scan_scalar_item'
      
      call scan_scalar(ic, typ)
      
      call verify_scan_current_char_and_advance(sbr, ';')
      c = int_to_char(10)
      call verify_scan_current_char(sbr, c)
      
      return
      end
      
      
      subroutine scan_scalar_items(ic, typ, sz)
C     Scan scalar items of an array.
      
C     Input arguments.
      integer ic, typ, sz
      
C     Local variable.
      integer k
      
      do k = 1, sz
         call scan_scalar_item(ic, typ)
      enddo
      
      return
      end
      
      
      subroutine scan_vector(ic, typ)
C     Scan a vector of values of type typ.
      
      implicit none
      
C     Input arguments.
      integer ic, typ
      
C     Local variables.
      character sbr*11
      integer item_typ, sz(1)
      
C     External function.
      logical pio_get_debug_mode
      integer scan_size
      
      sbr = 'scan_vector'
      
      item_typ = MAKE_VECTOR_ITEM_TYPE(typ)
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'typ = ', typ, 'item_typ = ', item_typ
      endif
      
      call verify_scan_next_line(sbr, ic, '[1', 2)
      call store_value_dimension(sbr, PIO_IN, VECTOR_DIMENSION)
      
      call fill_buffer(sbr, ic)
      sz(1) = scan_size()
      if (sz(1) .gt. MAX_VECTOR_SIZE) then
         call error_too_large_size(sbr, MAX_VECTOR_SIZE, sz(1))
      endif
      call store_value_size(sbr, PIO_IN, VECTOR_DIMENSION, sz)
      
      call scan_scalar_items(ic, item_typ, sz(1))
      
      call fill_buffer(sbr, ic)
      call verify_scan_current_string_and_advance(sbr, '1]', 2)
      
      return
      end
      
      
      subroutine scan_multi_d_sizes(pv, sizev)
C     Scan the sizes of a multi dimensional array value.
      
      implicit none
      
C     Input argument.
      integer pv
      
C     Output argument.
      integer sizev(*)
      
C     Local variable.
      character sbr*18
      
      sbr = 'scan_multi_d_sizes'
      
      call verify_scan_current_char_and_advance(sbr, '<')
      
      call scan_natural_list(sbr, pv, sizev)
      call store_value_size(sbr, PIO_IN, pv, sizev)
      
      call verify_scan_current_char(sbr, '>')
      
      return
      end
      
      
      subroutine scan_multi_d_layout(pv, layoutv)
C     Scan the layout of a multi dimensional array value.
      
      implicit none
      
C     Input arguments.
      integer pv, layoutv(*)
      
C     Local variables.
      character sbr*19, cur
      integer j, d
      
C     External functions.
      logical is_decimal_digit
      character get_scan_current_char
      integer char_to_int
      
      sbr = 'scan_multi_d_layout'
      
      cur = get_scan_current_char()
      
      if (cur .eq. 'C') then
         
         do j = 1, pv
            layoutv(j) = j - 1
         enddo
         
         call store_value_layout_kind(sbr, PIO_IN, C_LAYOUT_KIND)
         
      else if (cur .eq. 'F') then
         
         do j = 1, pv
            layoutv(j) = pv - j
         enddo
         
         call store_value_layout_kind(sbr, PIO_IN, FORTRAN_LAYOUT_KIND)
         
      else
         
         d = char_to_int(cur)
         
         if (is_decimal_digit(d)) then
         
            call store_value_layout_kind
     +           (sbr, PIO_IN, PERMUTATION_LAYOUT_KIND)
            call scan_natural_list(sbr, pv, layoutv)
            call store_value_layout(sbr, PIO_IN, pv, layoutv)
            
         else
            
            call error_unexpected_char(sbr, cur)
            
         endif
         
      endif
      
      return
      end
      
      
      subroutine scan_row(ic, typ, sz)
C     Scan a row of a multi dimensional array value.
      
      implicit none
      
C     Input arguments.
      integer ic, typ, sz
      
C     Local variable.
      character sbr*8
      
      sbr = 'scan_row'
      
      call verify_scan_next_line(sbr, ic, '[|', 2)
      
      call scan_scalar_items(ic, typ, sz)
      
      call fill_buffer(sbr, ic)
      call verify_scan_current_string_and_advance(sbr, '|]', 2)
      
      return
      end
      
      
      subroutine scan_row_item(ic, typ, sz)
C     Scan a row item of a multi dimensional array value.
      
      implicit none
      
C     Input arguments.
      integer ic, typ, sz
      
C     Local variables.
      character sbr*13, c
      
C     External function.
      character int_to_char
      
      sbr = 'scan_row_item'
      
      call scan_row(ic, typ, sz)
      
      call verify_scan_current_char_and_advance(sbr, ';')
      c = int_to_char(10)
      call verify_scan_current_char(sbr, c)
      
      return
      end
      
      
      subroutine scan_row_items(ic, typ, pv, sizev, layoutv)
C     Scan the rows of a multi dimensional array value.
      
      implicit none
      
C     Input arguments.
      integer ic, typ, pv, sizev(*), layoutv(*)
      
C     Local variable.
      character sbr*14
      integer nb_row, sz, j
      
      sbr = 'scan_row_items'
      
      call row_number_and_size
     +     (sbr, 'scanned', pv, sizev, layoutv, nb_row, sz)
      
      do j = 1, nb_row
         call scan_row_item(ic, typ, sz)
      enddo
      
      return
      end
      
      
      subroutine scan_multi_d(ic, typ)
C     Scan an multi dimensional array value.
      
      implicit none
      
C     Input arguments.
      integer ic, typ
      
C     Local variables.
      character sbr*12
      integer item_typ, pv, qv, sizev(MAX_ARRAY_DIMENSION)
      integer layoutv(MAX_ARRAY_DIMENSION)
      
C     External functions.
      logical pio_get_debug_mode
      integer scan_natural
      
      sbr = 'scan_multi_d'
      
      item_typ = MAKE_MULTI_D_ITEM_TYPE(typ)
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'typ = ', typ, 'item_typ = ', item_typ
      endif
      
      call fill_buffer(sbr, ic)
      call verify_scan_current_char_and_advance(sbr, '[')
      
      pv = scan_natural(sbr)
      call store_value_dimension(sbr, PIO_IN, MULTI_D_DIMENSION(pv))
      
      call fill_buffer(sbr, ic)
      call scan_multi_d_sizes(pv, sizev)
      call fill_buffer(sbr, ic)
      call scan_multi_d_layout(pv, layoutv)
      
      call scan_row_items(ic, item_typ, pv, sizev, layoutv)
      
      call fill_buffer(sbr, ic)
      qv = scan_natural(sbr)
      if (pv .ne. qv) then
         call error_incoherent_array_markers(sbr, pv, qv)
      endif
   
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'before verifying '']'''
      endif
      call verify_scan_current_char_and_advance(sbr, ']')
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'after verifying '']'''
      endif
      
      return
      end
      
      
      subroutine scan_array(ic, typ)
C     Scan an array value of type typ.
C     We already know that typ is not a scalar type.
      
      implicit none
      
C     Input arguments.
      integer ic, typ
      
C     External functions.
      logical is_vector_type
      
      if (is_vector_type(typ)) then
         call scan_vector(ic, typ)
      else
         call scan_multi_d(ic, typ)
      endif
      
      return
      end
      
      
      subroutine scan_array_item(ic, typ)
C     Scan an array item of type typ.
C     We already know that typ is not a scalar type.
      
      implicit none
      
C     Input arguments.
      integer ic, typ
      
C     Local variables.
      character sbr*15, c
      
C     External function.
      character int_to_char
      
      sbr = 'scan_array_item'
      
      call scan_array(ic, typ)
      
      call verify_scan_current_char_and_advance(sbr, ';')
      c = int_to_char(10)
      call verify_scan_current_char(sbr, c)
      
      return
      end
      
      
      subroutine scan_value(ic, typ)
C     Scan a value of type typ.
      
      implicit none
      
C     Input arguments.
      integer ic, typ
      
C     Local variables.
      character sbr*10
      integer bv, ev
      
C     External functions.
      logical pio_get_debug_mode, is_scalar_type
      integer get_store_ptr
      
      sbr = 'scan_value'
   
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'before getting bv'
      endif
      bv = get_store_ptr(PIO_IN)
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'after getting bv'
      endif
      
      if (is_scalar_type(typ)) then
         call store_value_dimension(sbr, PIO_IN, SCALAR_DIMENSION)
         call scan_scalar_item(ic, typ)
      else
         call scan_array_item(ic, typ)
      endif
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'before getting ev'
      endif
      ev = get_store_ptr(PIO_IN)
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'after getting ev'
      endif
      
      call store_value_begin(sbr, PIO_IN, bv)
      call store_value_end(sbr, PIO_IN, ev)
      call store_value_type(sbr, PIO_IN, typ)
      
      return
      end
      
      
      subroutine scan_value_begin(ic)
C     Scan the "begin" marker of a typed value.
      
      implicit none
      
C     Input argument.
      integer ic
      
C     Local variable.
      character sbr*16
      
C     External function.
      logical pio_get_debug_mode
      
      sbr = 'scan_value_begin'
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'before verifying begin'
      endif
      
      call verify_scan_next_line(sbr, ic, 'begin', 5)
      
      return
      end
      
      
      subroutine scan_value_end(ic)
C     Scan the "end" marker of a typed value.
      
      implicit none
      
C     Input argument.
      integer ic
      
C     Local variable.
      character sbr*14
      
C     External function.
      logical pio_get_debug_mode
      
      sbr = 'scan_value_end'
   
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'before verifying end'
      endif
      
      call verify_scan_next_line(sbr, ic, 'end', 3)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In ', sbr, ', ',
     +        'after verifying end'
      endif
      
C     Be prepared for the next value
      call next_value_next_value_descriptor(sbr, PIO_IN)
      
      return
      end
      
      
      subroutine scan_typed_value(ic)
C     Scan a typed value.
      
      implicit none
      
C     Input argument.
      integer ic
      
C     Local variables.
      character sbr*16
      integer typ
      
C     External functions.
      logical pio_get_debug_mode
      integer scan_value_type
      
      sbr = 'scan_typed_value'
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] In scan_typed_value'
      endif
      
      call scan_value_begin(ic)
      typ = scan_value_type(ic)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] type scanned'
      endif
      
      call scan_value(ic, typ)
      
      if (pio_get_debug_mode()) then
         write(PIO_DEBUG, *) '[Pio/Fortran] value scanned'
      endif
      
      call scan_value_end(ic)
      call verify_scan_empty_line(sbr, ic)
      
      return
      end
